function ER_MainTanksOnLoad()
	ER_mainTanksUpdate = 0;

	ER_MainTanksBuffsWidth = 0;

	ER_MainTanksAnchor_UpdateScale();
	ER_MainTanksAnchor_LoadLocation();

	ER_UpdateMainTanks();
end

function ER_MainTanksOnUpdate(elapsed)
	ER_mainTanksUpdate = ER_mainTanksUpdate + elapsed;
	if ( ER_mainTanksUpdate >= 1.5 ) then
		ER_UpdateMainTanks();
		ER_mainTanksUpdate = 0;
	end
end

function ER_MainTanksAnchor_InitLocation()
	ER_MainTanksAnchor:ClearAllPoints();
	ER_MainTanksAnchor:SetPoint("TOPRIGHT", "UIParent", "TOPLEFT", UIParent:GetWidth() - 210, -250);
	ER_MainTanksAnchor_SaveLocation();
end

function ER_MainTanksAnchor_LoadLocation()
	if ( ER_Config.mainTanksFrameLocation ) then
		ER_MainTanksAnchor:ClearAllPoints();
		ER_MainTanksAnchor:SetPoint(ER_Config.mainTanksEffectiveAlignment, "UIParent", "TOPLEFT",
									ER_Config.mainTanksFrameLocation.x, ER_Config.mainTanksFrameLocation.y);
	end
end

function ER_MainTanksAnchor_SaveLocation()
	ER_Config.mainTanksFrameLocation = { };
	local alignment = ER_GetSmartAlignment(ER_MainTanksAnchor, ER_Config.mainTanksAlignment);
	_, _, _, anchorX, anchorY = ER_MainTanksAnchor:GetPoint(1);
	ER_Config.mainTanksFrameLocation.x = anchorX;
	ER_Config.mainTanksFrameLocation.y = anchorY;
	ER_Config.mainTanksEffectiveAlignment = alignment;
	if ( ER_MainTanksAnchor:IsUserPlaced() ) then
		ER_MainTanksAnchor:SetUserPlaced(false);
	end
	ER_MainTanksAnchor_LoadLocation();
end

function ER_MainTanksAnchor_UpdateScale()
	ER_MainTanksAnchor:SetScale(ER_Config.mainTanksScale/100);
end

-----------------------------------------------------------------------

function ER_UpdateMainTanks()
	ER_mainTanks = { };

	local mts = ER_RaidAssistGetMainTanks();
	local i, name;
	local mainTanksHash;
	for i, name in pairs(mts) do
		ER_SetMainTank(i, name);
		mainTanksHash = (mainTanksHash or "").."["..i.."-"..name.."]";
	end

	if ( mainTanksHash ~= ER_mainTanksHash ) then
		ER_RaidGroups_DelayedUpdate();
		ER_RaidGroups_UpdateName();
		ER_MainTanksFrame_Update();
		ER_mainTanksHash = mainTanksHash;
	end
end

function ER_SetMainTank(number, name)
	ER_RemoveMainTank(name);

	local unit, index = ER_GetRaidUnitByName(name);
	if ( unit ) then
		ER_mainTanks[number] = { ["index"] = index, ["unit"] = unit, ["name"] = name };
	end
end

function ER_RemoveMainTank(name)
	local i, mt;		
	for i, mt in pairs(ER_mainTanks) do
		if ( mt.name == name ) then
			ER_mainTanks[i] = nil;
			break;
		end
	end
end

-----------------------------------------------------------------------
-- Main Tanks config methods

function ER_MainTanksFrame_OnUpdate(elapsed)
	if ( MOVING_RAID_MEMBER or ER_MOVING_MAIN_TANK ) then
		local i;
		ER_TARGET_MAIN_TANK_SLOT = nil;

		for i=1, ER_MAX_MAIN_TANKS do
			local slot = getglobal("ER_MainTanksFrameSlot"..i);
			if ( MouseIsOver(slot) ) then
				slot:LockHighlight();
				ER_TARGET_MAIN_TANK_SLOT = slot;
			else
				slot:UnlockHighlight();
			end
		end
	end
end

function ER_MainTanksFrame_Update()
	if ( not ER_MainTanksFrame:IsVisible() ) then
		return;
	end

	local i;

	for i=1, ER_MAX_MAIN_TANKS do
		local button = getglobal("ER_MainTankButton"..i);
		if ( ER_mainTanks and ER_mainTanks[i] ) then
			local mt = ER_mainTanks[i];
			local name, _, _, level, class, fileName, _, online, isDead = GetRaidRosterInfo(mt.index);

			local buttonName = getglobal("ER_MainTankButton"..i.."Name");
			local buttonClass = getglobal("ER_MainTankButton"..i.."Class");
			local buttonLevel = getglobal("ER_MainTankButton"..i.."Level");
			
			button.name = name;
			button.unit = mt.unit;
			button.class = fileName;
			
			SecureUnitButton_OnLoad(button, button.unit, RaidGroupButton_ShowMenu);
			
			if ( level == 0 ) then
				level = "";
			end
			
			if ( not name ) then
				name = UNKNOWN;
			end
			
			buttonName:SetText(name);

			if ( class ) then
				buttonClass:SetText(class);
			else
				buttonClass:SetText("");
			end
			
			if ( level ) then
				buttonLevel:SetText(level);
			else
				buttonLevel:SetText("");
			end
			
			if ( online ) then
				if ( isDead ) then
					buttonName:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
					buttonClass:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
					buttonLevel:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
				else
					color = RAID_CLASS_COLORS[fileName];
					if ( color ) then
						buttonName:SetVertexColor(color.r, color.g, color.b);
						buttonClass:SetVertexColor(color.r, color.g, color.b);
						buttonLevel:SetVertexColor(color.r, color.g, color.b);
					end
				end
			else
				buttonName:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
				buttonClass:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
				buttonLevel:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
			end
		
			-- Anchor button to slot
			if ( ER_MOVING_MAIN_TANK ~= button  ) then
				button:SetPoint("TOPLEFT", getglobal("ER_MainTanksFrameSlot"..i), "TOPLEFT", 0, 0);
				button:SetFrameLevel(ER_MainTanksFrame:GetFrameLevel()+2);
			end
			
			button:Show();
		else
			button:Hide();
		end
	end
end

function ER_MainTankButton_OnDragStart()
	if ( not (IsRaidLeader() or IsRaidOfficer()) ) then
		return;
	end
	local cursorX, cursorY = GetCursorPosition();
	this:ClearAllPoints();
	this:SetPoint("CENTER", "UIParent", "BOTTOMLEFT", cursorX/UIParent:GetScale(), cursorY/UIParent:GetScale());
	this:SetFrameLevel(ER_MainTanksFrame:GetFrameLevel()+3);
	this:StartMoving();
	ER_MOVING_MAIN_TANK = this;
end

function ER_MainTankButton_OnDragStop(mainTankButton)
	ER_MOVING_MAIN_TANK = nil;

	if ( not (IsRaidLeader() or IsRaidOfficer()) ) then
		return;
	end

	if ( not mainTankButton ) then
		mainTankButton = this;
	end
	
	mainTankButton:StopMovingOrSizing();

	mainTankButton:SetPoint("TOPLEFT", getglobal("ER_MainTanksFrameSlot"..mainTankButton:GetID()), "TOPLEFT", 0, 0);

	if ( ER_TARGET_MAIN_TANK_SLOT ) then
		ER_TARGET_MAIN_TANK_SLOT:UnlockHighlight();
		
		local number = ER_TARGET_MAIN_TANK_SLOT:GetID();
		local name = mainTankButton.name;
		
		if ( not ER_mainTanks[number] ) then
			ER_RaidAssistSetMainTank(number, name);
			ER_SetMainTank(number, name);
		else
			ER_RaidAssistSetMainTank(mainTankButton:GetID(), ER_mainTanks[number].name);
			ER_SetMainTank(mainTankButton:GetID(), ER_mainTanks[number].name);
			ER_RaidAssistSetMainTank(number, name);
			ER_SetMainTank(number, name);
		end
	else
		local name = mainTankButton.name;
		ER_RaidAssistRemoveMainTank(name);
		ER_RemoveMainTank(name);
	end

	ER_MainTanksFrame_Update();
	ER_mainTanksUpdate = 0;
end

function ER_RaidGroupFrame_OnUpdate(elapsed)
	if ( IsRaidLeader() ) then
		ER_SavedRaidGroupFrame_OnUpdate(elapsed);
	end
end

function ER_RaidGroupButton_OnDragStart(raidButton)
	if ( InCombatLockdown() ) then
		return;
	end

	if ( IsRaidOfficer() and not IsRaidLeader() ) then
		if ( not raidButton ) then
			raidButton = this;
		end		
	
		local cursorX, cursorY = GetCursorPosition();
		raidButton:StartMoving();
		raidButton:ClearAllPoints();
		raidButton:SetPoint("CENTER", nil, "BOTTOMLEFT", cursorX*GetScreenWidthScale(), cursorY*GetScreenHeightScale());
		MOVING_RAID_MEMBER = raidButton;
		SetRaidRosterSelection(raidButton.id);
	end
end

function ER_RaidGroupButton_OnDragStop(raidButton)
	if ( InCombatLockdown() ) then
		return;
	end

	if ( not ER_TARGET_MAIN_TANK_SLOT ) then
		if ( not IsRaidLeader() ) then
			MOVING_RAID_MEMBER = nil;

			if ( not raidButton ) then
				raidButton = this;
			end
			
			raidButton:StopMovingOrSizing();
	
			if ( TARGET_RAID_SLOT ) then
				TARGET_RAID_SLOT:UnlockHighlight();
			end
	
			raidButton:SetPoint("TOPLEFT", raidButton.slot, "TOPLEFT", 0, 0);
		end
	else
		MOVING_RAID_MEMBER = nil;

		if ( not (IsRaidLeader() or IsRaidOfficer()) ) then
			return;
		end

		if ( not raidButton ) then
			raidButton = this;
		end
		
		raidButton:StopMovingOrSizing();

		ER_TARGET_MAIN_TANK_SLOT:UnlockHighlight();

		raidButton:SetPoint("TOPLEFT", raidButton.slot, "TOPLEFT", 0, 0);
		
		local number = ER_TARGET_MAIN_TANK_SLOT:GetID();
		local name = raidButton.name;
		ER_RaidAssistSetMainTank(number, name);
		ER_SetMainTank(number, name);		

		ER_MainTanksFrame_Update();
		ER_mainTanksUpdate = 0;
	end
end
