function ER_RaidGroup_Update(frame)
	if ( not frame ) then
		frame = this;
	end

	local id = frame.filterID;

	if ( not id ) then
		return;
	end

	local numRaidMembers = GetNumRaidMembers();
	local playerGroupNumber;
	local playerIndex;
	local i, j;
	
	-- Find group number
	for i=1, numRaidMembers do
		local name, _, subgroup = GetRaidRosterInfo(i);
		if ( name == UnitName("player") ) then
			playerGroupNumber = subgroup;
			playerIndex = i;
			break;
		end
	end

	if ( not InCombatLockdown() ) then
		if ( id == "MAINTANKS" or id == "TARGETS" ) then
			RAID_SUBGROUP_LISTS[id] = { };
			if ( ER_mainTanks ) then
				local mt;
				for _, mt in ER_PairsByKeys(ER_mainTanks) do
					table.insert(RAID_SUBGROUP_LISTS[id], mt.index);
				end
			end
		end
	
		local groupList = RAID_SUBGROUP_LISTS[id];
		local cleanedGroupList = { };
		local listIsCleaned = false;
	
		local displayedGroups = { };

		if ( ER_Config.playerGroupIsShown and not ER_showAllGroups ) then
			displayedGroups[playerGroupNumber] = true;
		end
		
		local group;
		for _, group in pairs(ER_Config.groups) do
			if ( group.isShown or ER_showAllGroups ) then
				displayedGroups[group.number] = true;
			end
		end

		local unitIndex;

		if ( type(id) == "number" ) then
			-- Group
			if ( not displayedGroups[id] ) then
				listIsCleaned = true;
			elseif ( ER_Config.playerIsHidden and not ER_showAllGroups ) then
				-- Hide player
				for _, unitIndex in pairs(groupList) do
					if ( unitIndex ~= playerIndex ) then
						table.insert(cleanedGroupList, unitIndex);
					else
						listIsCleaned = true;
					end
				end
			end
		else
			-- Class
			local alreayDisplayedPlayers = { };
			
			if ( ER_Config.alreadyDisplayedPlayersAreHidden and id ~= "MAINTANKS" and id ~= "TARGETS" ) then				
				local groupNumber;
				for groupNumber in pairs(displayedGroups) do
					for _, unitIndex in pairs(RAID_SUBGROUP_LISTS[groupNumber]) do
						alreayDisplayedPlayers[unitIndex] = true;
					end
				end
				
				if ( ER_Config.mainTanksAreShown and ER_mainTanks ) then
					local mt;
					for _, mt in pairs(ER_mainTanks) do
						alreayDisplayedPlayers[mt.index] = true;
					end
				end
			end
			
			for _, unitIndex in pairs(groupList) do
				if ( id == "MAINTANKS" or id == "TARGETS" or (not alreayDisplayedPlayers[unitIndex] and not (ER_Config.playerIsHidden and unitIndex == playerIndex)) ) then
					table.insert(cleanedGroupList, unitIndex);
				else
					listIsCleaned = true;
				end
			end
		end
	
		if ( not listIsCleaned ) then
			cleanedGroupList = groupList;
		end
			
		local nameList;
		frame.numMembers = 0;	
		
		local numTargetsToShow = 0;
		local inCombat = UnitAffectingCombat("player");

		for i=1, getn(cleanedGroupList) do
			local unit = "raid"..cleanedGroupList[i];
			local unitName = UnitName(unit);
			if ( unitName ) then				
				if ( nameList ) then
					nameList = nameList .. ",";
				else
					nameList = "";
				end
		
				nameList = nameList .. unitName;

				frame.numMembers = frame.numMembers + 1;

				if ( id == "TARGETS" and (inCombat or (UnitExists(unit.."target") and
						(not ER_Config.showHostileTargetsOnly or UnitIsEnemy(unit.."target", "player")))) ) then
					numTargetsToShow = numTargetsToShow + 1;
				end
			end
		end

		if ( id == "TARGETS" and numTargetsToShow == 0 ) then
			nameList = nil;
			frame.numMembers = 0;
		end

		if ( frame:GetAttribute("nameList") ~= nameList ) then
			frame:SetAttribute("nameList", nameList);
		end

		if ( frame.numMembers > 0 ) then
			frame:Show();
			
			if ( ER_USE_PLAYER_GROUP_HIGHLIGHTING ) then
				if ( id == playerGroupNumber ) then
					getglobal(frame:GetName().."MenuBackdrop"):SetBackdropBorderColor(1, 1, 1);
				else
					getglobal(frame:GetName().."MenuBackdrop"):SetBackdropBorderColor(0.5, 0.5, 0.5);
				end
			end
		else
			frame:Hide();
			frame.filterID = nil;
		end
	end

	frame.serialNumber = ER_raidGroupSerialNumber;

	return frame.numMembers > 0;
end

function ER_RaidGroup_UpdateTargets()
	if ( ER_isLoaded ) then
		if ( ER_raidGroupTargets and ER_raidGroupTargets:IsShown() ) then
			local i;
			for i=1, ER_raidGroupTargets.numMembers do
				local button = getglobal(ER_raidGroupTargets:GetName().."UnitButton"..i);
				if ( button:IsShown() ) then
					if ( UnitExists(button.unit) and (not ER_Config.showHostileTargetsOnly or UnitIsEnemy(button.unit, "player")) ) then
						ER_UnitButton_UpdateName(button);
						ER_UnitButton_UpdateDead(button);

						local healthBar = getglobal(button:GetName().."HealthBar");
						UnitFrameHealthBar_Update(healthBar, button.unit);
						local _, hpMax = healthBar:GetMinMaxValues();
						local hp = healthBar:GetValue();
						local healthBarText = getglobal(button:GetName().."HealthBarText");
						healthBarText:SetText(floor(hp/hpMax*100).."%");
		
						local manaBar = getglobal(button:GetName().."ManaBar");
						UnitFrameManaBar_Update(manaBar, button.unit);

						ER_UpdateButtonHighlight(button);

						button.isHidden = nil;
					else
						button:SetAlpha(0);
						button.isHidden = true;
					end
				end
			end
		end
	end
end

-----------------------------------------------------------------------

function ER_RaidGroup_GenerateFrame(id)
	local i;
	local frame;
	
	-- Find a frame with filterID is id
	for i=1, ER_numRaidGroups do
		local searchFrame = getglobal("ER_RaidGroup"..i);
		
		if ( searchFrame.filterID == id ) then
			frame = searchFrame;
			break;
		end
	end

	-- Generate the frame if it doesn't exist
	if ( not frame ) then
		if ( type(id) == "number" ) then
			frame = ER_RaidGroup_GenerateGroupFrame(id);
		elseif ( id == "MAINTANKS" ) then
			frame = ER_RaidGroup_GenerateClassFrame("MTs", id);
		elseif ( id == "TARGETS" ) then
			frame = ER_RaidGroup_GenerateClassFrame(ER_TARGETS, id);
		else
			frame = ER_RaidGroup_GenerateClassFrame(ER_CLASSES_NAME[id], id);
		end
	else
		if ( not ER_RaidGroup_Update(frame) ) then
			frame = nil;
		end
	end
	
	if ( frame ) then
		-- Calculate buffs width
		local buffsWidth = 13;
		for i=1, frame.numMembers do
			local button = getglobal(frame:GetName().."UnitButton"..i);
			if ( button ) then
				local width = ceil((button.numBuffs or 0)/2) * 13;
				if ( width > buffsWidth ) then
					buffsWidth = width;
				end
			end
		end
		
		if ( id ~= "MAINTANKS" or (ER_Config.mainTanksAreAttached and ER_Config.mainTanksAttachment == "RAID") ) then
			if ( id ~= "TARGETS" ) then
				-- Set frame location
				frame:ClearAllPoints();
				if ( ER_previousRaidGroupNameY ) then				
					if ( getglobal(ER_previousRaidGroupNameY):GetBottom() - frame:GetHeight() + ER_raidGroupOffsetY - 24 > ER_ContainerFrame:GetBottom() / (ER_Config.scale/100) ) then
						frame:SetPoint("TOPLEFT", ER_previousRaidGroupNameY, "BOTTOMLEFT", 0, ER_raidGroupOffsetY - 12);
					else
						frame:SetPoint("TOPLEFT", ER_previousRaidGroupNameX, "TOPRIGHT", ER_raidGroupBuffsWidth + ER_RAID_GROUP_OFFSET_X, 0);
						
						ER_previousRaidGroupNameX = frame:GetName();
						ER_raidGroupBuffsWidth = buffsWidth;
					end
				else
					frame:SetPoint("TOPLEFT", "ER_ContainerFrame", "TOPLEFT", 4, -4+ER_raidGroupOffsetY);
					ER_previousRaidGroupNameX = frame:GetName();
					ER_firstRaidGroup = frame;
				end
							
				ER_previousRaidGroupNameY = frame:GetName();			
				if ( buffsWidth > ER_raidGroupBuffsWidth ) then
					ER_raidGroupBuffsWidth = buffsWidth;
				end
				
				local x = frame:GetRight() + ER_raidGroupBuffsWidth;
				if ( not ER_raidGroupMaxX or x > ER_raidGroupMaxX ) then
					ER_raidGroupMaxX = x;
				end
				local y = frame:GetBottom();
				if ( not ER_raidGroupMinY or y < ER_raidGroupMinY ) then
					ER_raidGroupMinY = y;
				end
				
				frame:SetScale(ER_Config.scale/100);
			else -- Targets frame positioning
				ER_TargetsAnchor:SetHeight(frame:GetHeight());
				frame:ClearAllPoints();
				local alignment = ER_TargetsAnchor:GetPoint();
				frame:SetScale(ER_TargetsAnchor:GetScale());
				frame:SetPoint(alignment, ER_TargetsAnchor, alignment);
			end
		else -- MTs are not attached to the raid
			if ( ER_Config.mainTanksAreAttached and ER_Config.mainTanksAttachment == "TARGETS" ) then
				ER_MainTanksBuffsWidth = buffsWidth;

				local alignment = ER_Config.targetsEffectiveAlignment;

				frame:ClearAllPoints();
				if ( ER_raidGroupTargets and ER_raidGroupTargets:IsShown() ) then
					if ( alignment == "TOPLEFT" or alignment == "BOTTOMLEFT" ) then
						frame:SetPoint("TOPRIGHT", ER_TargetsAnchor, "TOPLEFT", 0 - ER_MainTanksBuffsWidth - ER_RAID_GROUP_OFFSET_X, 0);
					else
						frame:SetPoint("TOPRIGHT", ER_TargetsAnchor, "TOPLEFT", 0 - ER_MainTanksBuffsWidth - ER_RAID_GROUP_OFFSET_X - 10, 0);
					end
				else
					frame:SetPoint("TOPRIGHT", ER_TargetsAnchor, "TOPRIGHT", 0, 0);
				end
				
				frame:SetScale(ER_TargetsAnchor:GetScale());

				if ( alignment == "TOPLEFT" or alignment == "BOTTOMLEFT" ) then
					ER_TargetsAnchor_LoadLocation();
				end
			else
				ER_MainTanksAnchor:SetHeight(frame:GetHeight());
				frame:ClearAllPoints();
				local alignment = ER_MainTanksAnchor:GetPoint();
				frame:SetScale(ER_MainTanksAnchor:GetScale());
				frame:SetPoint(alignment, ER_MainTanksAnchor, alignment);
			end
		end		
	end
	
	return frame;
end

function ER_RaidGroup_GetFrame(filterID)
	-- Grab an available frame
	local frame;
	for i=1, ER_numRaidGroups do 
		frame = getglobal("ER_RaidGroup"..i);
		-- if frame is visible see if its group id is already taken
		if ( frame:IsVisible() and filterID == frame.filterID ) then
			return nil;
		end
	end
	for i=1, ER_numRaidGroups do
		frame = getglobal("ER_RaidGroup"..i);
		if ( not frame:IsVisible() ) then
			return frame;
		end
	end

	ER_numRaidGroups = ER_numRaidGroups + 1;

	frame = CreateFrame("Button", "ER_RaidGroup"..ER_numRaidGroups, UIParent, "ER_RaidGroupTemplate");
	frame:SetAttribute("template","ER_UnitButtonTemplate");
	frame:SetAttribute("minHeight", 10);
	frame.initialConfigFunction = function(button) ER_UnitButton_Init(button); end

	if ( not ER_Config.titleOfTheFramesIsShown ) then
		getglobal(frame:GetName().."Name"):Hide();
		ER_raidGroupOffsetY = 0;
	else
		getglobal(frame:GetName().."Name"):Show();
		ER_raidGroupOffsetY = -12;
	end

	return frame;
end

function ER_RaidGroup_GenerateGroupFrame(groupID)
	-- construct the button listing
	if ( not groupID ) then
		groupID = this:GetParent():GetID();
	end
	
	-- Get a handle on a frame
	local frame = ER_RaidGroup_GetFrame(groupID);
	if ( frame ) then
		frame.filterID = groupID;
		-- Set name
		getglobal(frame:GetName().."Name"):SetText(GROUP.." "..groupID);
		if ( ER_RaidGroup_Update(frame) ) then
			return frame;		
		end
	end
end

function ER_RaidGroup_GenerateClassFrame(class, fileName)
	-- construct the button listing
	if ( not class ) then
		class = this:GetParent().class;
	end
	
	-- Get a handle on a frame
	local frame = ER_RaidGroup_GetFrame(fileName);
	if ( frame ) then
		frame.filterID = fileName;
		-- Set name
		getglobal(frame:GetName().."Name"):SetText(class);
		if ( ER_RaidGroup_Update(frame) ) then
			return frame;		
		end
	end
end

-----------------------------------------------------------------------

function ER_RaidGroup_OnDragStart(frame)
	if ( not frame ) then
		frame = this;
	end

	if ( frame.filterID ~= "MAINTANKS" or (ER_Config.mainTanksAreAttached and ER_Config.mainTanksAttachment == "RAID") ) then
		if ( frame.filterID ~= "TARGETS" ) then
			if ( not ER_ContainerFrame:IsShown() and not InCombatLockdown() ) then
				UIFrameFadeIn(ER_ContainerFrame, 1, 0, ER_CONTAINER_FRAME_ALPHA);
			end
			ER_ContainerFrame_StartMoving();
		else
			ER_TargetsFrameIsMovingOrSizing = true;
			ER_TargetsAnchor:StartMoving();
		end
	else
		if ( ER_Config.mainTanksAreAttached and ER_Config.mainTanksAttachment == "TARGETS" ) then
			ER_TargetsFrameIsMovingOrSizing = true;
			ER_TargetsAnchor:StartMoving();
		else
			ER_MainTanksAnchor:StartMoving();
		end
	end
end

function ER_RaidGroup_OnDragStop(frame)
	if ( not frame ) then
		frame = this;
	end

	if ( frame.filterID ~= "MAINTANKS" or (ER_Config.mainTanksAreAttached and ER_Config.mainTanksAttachment == "RAID") ) then
		if ( frame.filterID ~= "TARGETS" ) then
			ER_ContainerFrame_StopMoving();
		else
			ER_TargetsAnchor:StopMovingOrSizing();
			ER_TargetsFrameIsMovingOrSizing = false;
			ER_SetAnchorPoint(ER_TargetsAnchor, ER_Config.targetsAlignment);
			ER_TargetsAnchor_SaveLocation();
			ER_RaidGroups_DelayedUpdate();
		end
	else
		if ( ER_Config.mainTanksAreAttached and ER_Config.mainTanksAttachment == "TARGETS" ) then
			ER_TargetsAnchor:StopMovingOrSizing();
			ER_TargetsFrameIsMovingOrSizing = false;
			ER_SetAnchorPoint(ER_TargetsAnchor, ER_Config.targetsAlignment);
			ER_TargetsAnchor_SaveLocation();
			ER_RaidGroups_DelayedUpdate();
		else
			ER_MainTanksAnchor:StopMovingOrSizing();
			ER_SetAnchorPoint(ER_MainTanksAnchor, ER_Config.mainTanksAlignment);
			ER_MainTanksAnchor_SaveLocation();
			ER_RaidGroups_DelayedUpdate();
		end
	end
end