-- Display Main Tanks targets

function ER_TargetsOnLoad()
	ER_aggroUpdate = 0;
	ER_pulsingFramesUpdate = 0;
	ER_iconsUpdate = 0;	
	ER_raidTargetIcons = { };

	ER_InitPulsingFrames();
	ER_TargetsAnchor_UpdateScale();
	ER_TargetsAnchor_LoadLocation();
end

function ER_TargetsOnUpdate(elapsed)
	ER_aggroUpdate = ER_aggroUpdate + elapsed;
	if ( ER_aggroUpdate >= 0.5 ) then
		ER_UpdateAggro();
		ER_aggroUpdate = 0;
	end

	ER_pulsingFramesUpdate = ER_pulsingFramesUpdate + elapsed;
	if ( ER_pulsingFramesUpdate >= 0.3 ) then
		ER_UpdatePulsingFrames();
		ER_pulsingFramesUpdate = 0;
	end

	ER_iconsUpdate = ER_iconsUpdate + elapsed;
	if ( ER_iconsUpdate >= 0.5 ) then
		ER_UpdateIcons();
		ER_iconsUpdate = 0;
	end
end

function ER_TargetsAnchor_InitLocation()
	ER_TargetsAnchor:ClearAllPoints();
	ER_TargetsAnchor:SetPoint("TOPRIGHT", "UIParent", "TOPLEFT", UIParent:GetWidth() - 100, -250);
	ER_TargetsAnchor_SaveLocation();
end

function ER_TargetsAnchor_LoadLocation()
	if ( ER_Config.targetsFrameLocation ) then
		local alignment = ER_Config.targetsEffectiveAlignment;
		
		local offsetX = 0;
		if ( ER_Config.mainTanksAreAttached and ER_Config.mainTanksAttachment == "TARGETS"
				and (alignment == "TOPLEFT" or alignment == "BOTTOMLEFT") ) then
			offsetX = ER_MainTanksBuffsWidth;
		end
		
		ER_TargetsAnchor:ClearAllPoints();
		ER_TargetsAnchor:SetPoint(alignment, "UIParent", "TOPLEFT", ER_Config.targetsFrameLocation.x + offsetX, ER_Config.targetsFrameLocation.y);
	end
end

function ER_TargetsAnchor_SaveLocation()
	ER_Config.targetsFrameLocation = { };
	local alignment = ER_GetSmartAlignment(ER_TargetsAnchor, ER_Config.targetsAlignment);
	_, _, _, anchorX, anchorY = ER_TargetsAnchor:GetPoint(1);
	ER_Config.targetsFrameLocation.x = anchorX;
	ER_Config.targetsFrameLocation.y = anchorY;
	ER_Config.targetsEffectiveAlignment = alignment;
	if ( ER_TargetsAnchor:IsUserPlaced() ) then
		ER_TargetsAnchor:SetUserPlaced(false);
	end
	ER_TargetsAnchor_LoadLocation();
end

function ER_TargetsAnchor_UpdateScale()
	ER_TargetsAnchor:SetScale(ER_Config.targetsScale/100);
end

-----------------------------------------------------------------------

function ER_PlayerTargetChanged()
	if ( not ER_isLoaded or not ER_RaidAssistIsAvailable() ) then
		return;
	end

	local i;
	for i=1, ER_numRaidGroups do
		local frame = getglobal("ER_RaidGroup"..i);
		if ( frame:IsShown() ) then
			local j;
			for j=1, frame.numMembers do
				local button = getglobal("ER_RaidGroup"..i.."UnitButton"..j);
				if ( button and button:IsShown() ) then
					ER_UpdateButtonHighlight(button);
				end
			end
		end
	end
end

-----------------------------------------------------------------------

function ER_UpdateButtonHighlight(button)
	if ( UnitIsUnit("target", button.unit) ) then
		if ( not button.isHighlighted ) then
			button:SetBackdrop({bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
					insets = { left = 5, right = 5, top = 5, bottom = -7 }});
			button:SetBackdropColor(1.0, 1.0, 1.0, 0.2);
			button.isHighlighted = true;
		end
	else
		if ( button.isHighlighted ) then
			button:SetBackdrop(nil);
			button.isHighlighted = false;
		end
	end
end

-----------------------------------------------------------------------

function ER_UpdateAggro()
	local targets = { };
	local numTargets = 0;
	local unit, unitTarget, target, targetTarget;
	local frame, button;
	local i, j;

	for i = 1, GetNumRaidMembers() do
		unit = "raid"..i;
		unitTarget = unit.."target";
		if ( UnitExists(unitTarget) and UnitIsEnemy(unit, unitTarget) ) then
			local isUnique = true;
			for _, target in pairs(targets) do
				if ( UnitIsUnit(target, unitTarget) ) then
					isUnique = false;
					break;
				end
			end
			if ( isUnique ) then
				numTargets = numTargets + 1;
				targets[numTargets] = unitTarget;
			end
		end
	end
	
	ER_ResetPulsingFrames();
	
	for _, target in pairs(targets) do
		targetTarget = target.."target";
		
		if ( ER_Config.highlightAttackedPlayers ) then
			for i=1, ER_numRaidGroups do
				frame = getglobal("ER_RaidGroup"..i);
				if ( frame:IsShown() ) then
					for j=1, frame.numMembers do
						button = getglobal("ER_RaidGroup"..i.."UnitButton"..j);
						if ( button:IsShown() and UnitIsUnit(button.unit, targetTarget) ) then
							ER_AddPulsingFrame(button);
						end
					end
				end
			end
		end

		if ( ER_Config.soundsOnAggro and UnitIsUnit(targetTarget, "player") ) then
			if ( ER_playerHasAggro == nil ) then
				ER_AggroAlert();
			end
			ER_playerHasAggro = true;
		end
	end
	
	if ( ER_playerHasAggro ~= nil ) then
		if ( not ER_playerHasAggro ) then
			ER_playerHasAggro = nil;
		else
			ER_playerHasAggro = false;
		end
	end	
end

function ER_InitPulsingFrames()
	ER_pulsingFrames = { };
end

function ER_AddPulsingFrame(button)
	ER_pulsingFrames[button] = true;
end

function ER_ResetPulsingFrames()
	local button;
	for button in pairs(ER_pulsingFrames) do
		ER_pulsingFrames[button] = false;
	end
end

function ER_UpdatePulsingFrames()
	local button, buttonFrame;

	ER_pulseState = not ER_pulseState;

	for button in pairs(ER_pulsingFrames) do
		buttonFrame = getglobal(button:GetName().."Frame");
		if ( ER_pulsingFrames[button] == true ) then
			if ( ER_pulseState ) then
				buttonFrame:SetVertexColor(1, 0, 0, 1);
				buttonFrame:SetBlendMode("ADD");
			else
				buttonFrame:SetVertexColor(1, 1, 1, 0.2);
				buttonFrame:SetBlendMode("BLEND");
			end
		else
			buttonFrame:SetVertexColor(1, 1, 1, 1);
			buttonFrame:SetBlendMode("BLEND");
			ER_pulsingFrames[button]  = nil;
		end
	end
end

function ER_AggroAlert()
	if ( ER_RaidAssistHasMTSupport() and ER_Config.soundsOnAggro ) then
		local time = GetTime();
		if ( not ER_lastAggroAlertTime or time - ER_lastAggroAlertTime >= 1 ) then
			PlaySoundFile("Interface\\AddOns\\EasyRaid\\Sounds\\AggroAlert.wav");
			ER_lastAggroAlertTime = time;
		end
	end
end

-----------------------------------------------------------------------

function ER_UpdateIcons()
	if ( not (ER_Config.automaticallySetTargetsIcons and (IsRaidLeader() or IsRaidOfficer())) ) then
		return;
	end

	if ( GetRaidTargetIndex("player") ) then
		SetRaidTarget("player", 0);
	end

	local i = 0;
	local index = 1;
	local mt, target, mtHasUniqueTarget, currentIndex;
	local numMTs = 0;
	
	for _, _ in pairs(ER_mainTanks) do
		numMTs = numMTs + 1;
	end
	
	for _, mt in pairs(ER_mainTanks) do
		if ( numMTs - i <= 8 ) then -- Skip first MTs if they are more than 8
			-- if ( GetRaidTargetIndex(mt.unit) ) then
			--	SetRaidTarget(mt.unit, 0); -- Remove icons on MTs
			-- end
			
			target = mt.unit.."target";
			mtHasUniqueTarget = false;
			if ( UnitExists(target) and (not UnitInRaid(target)) ) then
				currentIndex = GetRaidTargetIndex(target) or 0;
				if ( currentIndex == 0 or currentIndex == index or (currentIndex > index and not UnitAffectingCombat(mt.unit)) ) then
					if ( currentIndex ~= index ) then
						SetRaidTarget(target, index);
						ER_raidTargetIcons[index] = true;
						if ( currentIndex ~= 0 ) then
							ER_raidTargetIcons[currentIndex] = nil;
						end
					end
					mtHasUniqueTarget = true;
				end
			end
			
			if ( not mtHasUniqueTarget ) then
				if ( ER_raidTargetIcons[index] ) then
					-- SetRaidTarget(mt.unit, index); -- Temporary put the icon on the MT and clear it later
					SetRaidTarget("player", index);
					ER_raidTargetIcons[index] = nil;
				end
			end
			
			index = index + 1;
		end		
		i = i + 1;
	end
end