function ER_UnitButton_Init(button)
	if ( button ) then
		button:SetAttribute("initial-width", 94)
		button:SetAttribute("initial-height", ER_UNIT_BUTTON_HEIGHT )		
		-- button:SetAttribute("initial-scale",self.scale)		
		-- button:SetAttribute("initial-unitWatch",true)

		button:SetAttribute("type1", "target");
		button:SetAttribute("type2", "menu");
		button.menu = ER_UnitButton_ShowMenu;	

		-- set up click casting
		ClickCastFrames = ClickCastFrames or {};
		ClickCastFrames[button] = true;
	end
end

function ER_UnitButton_OnEvent()
	if ( this:IsShown() ) then
		if ( event == "UNIT_HEALTH" ) then
			if ( arg1 == this.unit ) then
				if ( UnitIsConnected(this.unit) ) then
					ER_UnitButton_UpdateDead(this);
				end
			end
		elseif ( event == "UNIT_AURA" ) then
			if ( arg1 == this.unit ) then
				ER_UnitButton_UpdateBuffs(this);
			end
		end
	end
end

function ER_UnitButton_Update(button)
	local unit = button.unit;

	if ( unit ) then
		button.isHidden = nil;

		ER_UnitButton_UpdateName(button);

		local healthBar = getglobal(button:GetName().."HealthBar");
		local manaBar = getglobal(button:GetName().."ManaBar");
		button.manabar = manaBar;
		manaBar.unit = unit;
		UnitFrameHealthBar_Initialize(unit, healthBar);
		UnitFrameHealthBar_Update(healthBar, unit);
		UnitFrameManaBar_Initialize(unit, manaBar);
		UnitFrameManaBar_Update(manaBar, unit);

		ER_UnitButton_UpdateDead(button);
		ER_UnitButton_UpdateAFK(button);
		ER_UnitButton_UpdatePVPFlag(button);
		ER_UnitButton_UpdateRaidTargetIcon(button);
		ER_UnitButton_UpdateRangeCheck(button);
		ER_UnitButton_UpdateBuffs(button);
		ER_UnitButton_UpdateWidth(button);

 		button:RegisterEvent("UNIT_HEALTH");
		button:RegisterEvent("UNIT_AURA");
	else
		button:UnregisterAllEvents();
	end
end

function ER_UnitButton_UpdateName(button)
	if ( button.unit ) then
		local name, server;
		if ( UnitExists(button.unit) ) then
			name, server = UnitName(button.unit);
			if ( server and false ) then -- 
				name = name.." (*)";
			end
		end

		if ( name and ER_mainTanks ) then
			if ( not (ER_Config.mainTanksAreShown and button:GetParent().filterID ~= "MAINTANKS" and button:GetParent().filterID ~= "TARGETS") and 
					not (button:GetParent().filterID == "TARGETS" and ER_Config.mainTanksAreShown and ER_Config.mainTanksAreAttached and ER_Config.mainTanksAttachment == "TARGETS") ) then
				local mt, mtNumber;
				for mtNumber, mt in pairs(ER_mainTanks) do
					if ( mt.index == button.raidIndex ) then
						if ( UnitIsConnected(button.unit) ) then
							if ( UnitIsFriend(button.unit, "player") ) then
								name = "|c0000ff00"..mtNumber..".|r".." "..name;
							elseif ( UnitIsEnemy(button.unit, "player") ) then
								name = "|c00ff0000"..mtNumber..".|r".." "..name;
							else
								name = "|c00ffff00"..mtNumber..".|r".." "..name;
							end
						else
							name = mtNumber.." "..name;
						end
						break;
					end
				end
			end
		end

		getglobal(button:GetName().."Name"):SetText(name);
	end
end

function ER_UnitButton_UpdateAFK(button)
	if ( button:GetParent().filterID ~= "TARGETS" ) then
		local healthBarText = getglobal(button:GetName().."HealthBarText");
		if ( UnitIsAFK(button.unit) ) then
			healthBarText:SetText(CHAT_MSG_AFK);
		else
			healthBarText:SetText(nil);
		end
	end
end

function ER_UnitButton_UpdateRaidTargetIcon(button)
	local raidTargetIndex;

	if ( not button.haveFlag ) then
		if ( button:GetParent().filterID == "MAINTANKS" and not UnitAffectingCombat("player") ) then
			if ( not (ER_Config.mainTanksAreAttached and ER_Config.mainTanksAttachment == "TARGETS") ) then
				if ( UnitExists(button.unit.."target") and not UnitInRaid(button.unit.."target") ) then
					raidTargetIndex = GetRaidTargetIndex(button.unit.."target");
				end
			end
		end
		
		if ( not raidTargetIndex ) then
			if ( UnitExists(button.unit) ) then
				raidTargetIndex = GetRaidTargetIndex(button.unit);
			end
		else
			raidTargetIndex = -raidTargetIndex;
		end
	end

	if ( button.raidTargetIndex ~= raidTargetIndex ) then
		local raidTargetIcon = getglobal(button:GetName().."RaidTargetIcon");
		if ( raidTargetIndex ) then
			if ( not raidTargetIcon ) then
				raidTargetIcon = button:CreateTexture(button:GetName().."RaidTargetIcon", "ARTWORK");
				raidTargetIcon:SetWidth(12);
				raidTargetIcon:SetHeight(12);
				raidTargetIcon:SetPoint("TOPRIGHT", button:GetName().."Frame", "TOPLEFT", -2, -1);
				raidTargetIcon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons");
			end
			
			SetRaidTargetIconTexture(raidTargetIcon, abs(raidTargetIndex));

			if ( raidTargetIndex > 0 ) then
				raidTargetIcon:SetVertexColor(1, 1, 1, 1);
			else
				raidTargetIcon:SetVertexColor(0.6, 0.6, 0.6, 1);
			end
			
			raidTargetIcon:Show();
			ER_UnitButton_UpdateWidth(button);
		else
			if ( raidTargetIcon and raidTargetIcon:IsShown() ) then
				raidTargetIcon:Hide();
				ER_UnitButton_UpdateWidth(button);
			end
		end
		
		button.raidTargetIndex = raidTargetIndex;
	end
end

function ER_UnitButton_UpdatePVPFlag(button)
	local haveFlag;

	local _, instanceType = IsInInstance();
	if ( instanceType == "pvp" ) then
		local buff;
		local i = 1;
		repeat
			_, _, buff = UnitBuff(button.unit, i);
			local _, _, flag = string.find(buff or "", "INV_BannerPVP_0(%d)");
			if ( flag ) then
				haveFlag = tonumber(flag);
			end
			i = i + 1;
		until haveFlag or not buff;
	end		

	if ( button.haveFlag ~= haveFlag ) then
		local flagIcon = getglobal(button:GetName().."FlagIcon");
		local flagFlash = getglobal(button:GetName().."FlagFlash");
		local flagFlashTexture = getglobal(button:GetName().."FlagFlashTexture");
		if ( haveFlag ) then
			if ( not flagIcon ) then
				flagIcon = button:CreateTexture(button:GetName().."FlagIcon", "ARTWORK");
				flagIcon:SetWidth(16);
				flagIcon:SetHeight(16);
				flagIcon:SetPoint("TOPRIGHT", button:GetName().."Frame", "TOPLEFT", 0, 1);
				flagFlash = CreateFrame("Frame", button:GetName().."FlagFlash", button);
				flagFlash:SetWidth(16);
				flagFlash:SetHeight(16);
				flagFlash:SetPoint("TOPRIGHT", button:GetName().."Frame", "TOPLEFT", 0, 1);
				flagFlashTexture = flagFlash:CreateTexture(button:GetName().."FlagFlashTexture", "ARTWORK");
				flagFlashTexture:SetWidth(16);
				flagFlashTexture:SetHeight(16);
				flagFlashTexture:SetPoint("CENTER", 0, 0);
				flagFlashTexture:SetBlendMode("ADD");
			end
			
			if ( haveFlag == 1 ) then
				flagIcon:SetTexture("Interface\\WorldStateFrame\\HordeFlag");
				flagFlashTexture:SetTexture("Interface\\WorldStateFrame\\HordeFlagFlash");
			else
				flagIcon:SetTexture("Interface\\WorldStateFrame\\AllianceFlag");
				flagFlashTexture:SetTexture("Interface\\WorldStateFrame\\AllianceFlagFlash");
			end
			
			flagIcon:Show();
			flagFlash:Show();
			UIFrameFlash(flagFlash, 0.5, 0.5, -1);
			ER_UnitButton_UpdateWidth(button);
		else
			if ( flagIcon and flagIcon:IsShown() ) then
				flagIcon:Hide();
				UIFrameFlashStop(flagFlash);
				flagFlash:Hide();
				ER_UnitButton_UpdateWidth(button);
			end
		end
		
		button.haveFlag = haveFlag;
	end
end

function ER_UnitButton_UpdateRangeCheck(button)
	if ( button.isHidden or not UnitExists(button.unit) ) then
		return;
	end

	if ( ER_UnitIsInRange(button.unit) ) then
		button:SetAlpha(1);
		getglobal(button:GetName().."Frame"):SetAlpha(1);
		getglobal(button:GetName().."HealthBar"):SetAlpha(1);
		getglobal(button:GetName().."ManaBar"):SetAlpha(1);
	else
		button:SetAlpha(0.8);
		getglobal(button:GetName().."Frame"):SetAlpha(0.5);
		getglobal(button:GetName().."HealthBar"):SetAlpha(0.3);
		getglobal(button:GetName().."ManaBar"):SetAlpha(0.3);
	end

	if ( CheckInteractDistance(button.unit, 4) ) then
		button.debuffIsOutOfRange = nil;
	else
		button.debuffIsOutOfRange = true;
	end

	if ( not button.debuffIsOutOfRange ) then
		getglobal(button:GetName().."Debuff1"):SetAlpha(1);
	else
		getglobal(button:GetName().."Debuff1"):SetAlpha(0.7);
	end

	if ( getglobal(button:GetName().."Debuff1Border"):IsShown() and not button.debuffIsOutOfRange ) then
		if ( not button.debuffIsFlashing ) then
			UIFrameFlash(getglobal(button:GetName().."Debuff1Border"), 0.3, 0.3, -1);
			button.debuffIsFlashing = true;
		end
	else
		if ( button.debuffIsFlashing ) then
			UIFrameFlashStop(getglobal(button:GetName().."Debuff1Border"));
			button.debuffIsFlashing = nil;
		end
	end
end

function ER_UnitButton_UpdateDead(button)
	if ( UnitExists(button.unit) ) then
		local buttonName = getglobal(button:GetName().."Name");
		local unitIsRessing;

		if ( UnitIsConnected(button.unit) ) then
			local isReallyDead;
			if ( UnitInRaid(button.unit) ) then
				_, _, _, _, _, _, _, _, isReallyDead = GetRaidRosterInfo(button.raidIndex);
				if ( not isReallyDead ) then
					isReallyDead = not UnitIsDead(button.unit) and UnitIsDeadOrGhost(button.unit);
				end
			else
				isReallyDead = UnitIsDeadOrGhost(button.unit);
			end
			
			if ( isReallyDead ) then
				buttonName:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
				if ( ER_RaidAssistGetUnitIsRessing(button.unit) ) then
					unitIsRessing = true;
					if ( not buttonName.isFlashing ) then
						UIFrameFlash(buttonName, 0.75, 0.25, -1, 1);
						buttonName.isFlashing = true;
					end
				end
			else
				local _, class = UnitClass(button.unit);
				local color = RAID_CLASS_COLORS[class];
				if ( color ) then
					buttonName:SetVertexColor(color.r, color.g, color.b);
				end
			end
		else
			local healthBar = getglobal(button:GetName().."HealthBar");
			local manaBar = getglobal(button:GetName().."ManaBar");
			local _, hpMax = healthBar:GetMinMaxValues();
			healthBar:SetValue(hpMax);
			healthBar:SetStatusBarColor(0.5, 0.5, 0.5);
			buttonName:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
			UnitFrameManaBar_Update(manaBar, unit);
		end
		
		if ( buttonName.isFlashing and not unitIsRessing ) then
			buttonName.flashDuration = 0;
			buttonName.isFlashing = nil;
		end
	end
end

function ER_UnitButton_UpdateBuffs(button)
	local unit = button.unit;
	local inCombat = UnitAffectingCombat("player");

	local castableBuffs = { };
	local otherBuffs = { };
	local i;
	local buff, count;

	if ( button:GetParent().filterID ~= "TARGETS" ) then
		i = 1;
		repeat
			_, _, buff, count = UnitBuff(unit, i, 1);
			if ( count == 0 ) then
				count = 1;
			end
			if ( buff ) then
				buff = string.sub(buff, string.len(ER_INTERFACE_ICON_PATH) + 1);
				if ( not (inCombat and ER_commonBuffs[buff]) ) then
					if ( not castableBuffs[buff] ) then
						castableBuffs[buff] = { ["index"] = 12345, ["count"] = count };
					else
						castableBuffs[buff].count = castableBuffs[buff].count + count;
					end
				end
				i = i + 1;
			end
		until not buff;
		
		i = 1;
		repeat
			_, _, buff, count = UnitBuff(unit, i);
			if ( count == 0 ) then
				count = 1;
			end
			if ( buff ) then
				buff = string.sub(buff, string.len(ER_INTERFACE_ICON_PATH) + 1);
				if ( not (inCombat and ER_commonBuffs[buff]) ) then
					if ( castableBuffs[buff] ) then
						castableBuffs[buff].index = i;
					elseif ( SHOW_CASTABLE_BUFFS == "0" and false ) then -- *** TEMPORARY DISABLED ***
						if ( not otherBuffs[buff] ) then
							otherBuffs[buff] = { ["index"] = i, ["count"] = count };
						else
							otherBuffs[buff].count = otherBuffs[buff].count + count;
						end
					end
				end
				i = i + 1;
			end
		until not buff;
	end

	local maxBuffs;
	if ( not inCombat ) then
		maxBuffs = ER_MAX_BUFFS;
	else
		maxBuffs = 2;
	end

	local value;
	i = 1;

	for buff, value in pairs(castableBuffs) do
		if ( i > maxBuffs ) then
			break;
		end
		getglobal(button:GetName().."Buff"..i.."Icon"):SetTexture(ER_INTERFACE_ICON_PATH..buff);
		getglobal(button:GetName().."Buff"..i):Show();
		getglobal(button:GetName().."Buff"..i).index = value.index;
		if ( value.count > 1 ) then
			getglobal(button:GetName().."Buff"..i.."Count"):SetText(value.count);
		else
			getglobal(button:GetName().."Buff"..i.."Count"):SetText(nil);
		end
		i = i + 1;
	end
	
	for buff, value in pairs(otherBuffs) do
		if ( i > maxBuffs ) then
			break;
		end
		getglobal(button:GetName().."Buff"..i.."Icon"):SetTexture(ER_INTERFACE_ICON_PATH..buff);
		getglobal(button:GetName().."Buff"..i):Show();
		getglobal(button:GetName().."Buff"..i).index = value.index;
		if ( value.count > 1 ) then
			getglobal(button:GetName().."Buff"..i.."Count"):SetText(value.count);
		else
			getglobal(button:GetName().."Buff"..i.."Count"):SetText(nil);
		end
		i = i + 1;
	end
	
	button.numBuffs = i - 1;
	
	for i = button.numBuffs + 1, ER_MAX_BUFFS do
		getglobal(button:GetName().."Buff"..i):Hide();
	end

	local debuff, debuffIsDispellable;

	if ( button:GetParent().filterID ~= "TARGETS" ) then
		_, _, debuff  = UnitDebuff(unit, 1, 1);
		if ( debuff ) then
			debuffIsDispellable = true;
		elseif ( SHOW_DISPELLABLE_DEBUFFS == "0" or true ) then -- *** TEMPORARY ENABLED ***
			_, _, debuff = UnitDebuff(unit, 1);
		end
	end

	if ( debuff ) then
		getglobal(button:GetName().."Debuff1").unit = unit;
		getglobal(button:GetName().."Debuff1").isDispellable = debuffIsDispellable;

		if ( not button.debuff ) then		
			getglobal(button:GetName().."Debuff1"):Show();
			ER_UnitButton_UpdateWidth(button);
		end
	
		getglobal(button:GetName().."Debuff1Icon"):SetTexture(debuff);
		getglobal(button:GetName().."Debuff1"):SetScale(button:GetParent():GetScale());
		
		if ( CheckInteractDistance(unit, 4) ) then
			button.debuffIsOutOfRange = nil;
		else
			button.debuffIsOutOfRange = true;
		end

		count = 0;
		i = 1;
		local debuff2, count2;
		repeat
			_, _, debuff2, count2 = UnitDebuff(unit, i);
			if ( count2 == 0 ) then
				count2 = 1;
			end
			if ( debuff2 ) then
				if ( debuff2 == debuff ) then
					count = count + count2;
				end
				i = i + 1;
			end
		until not debuff2;

		if ( count > 1 ) then
			getglobal(button:GetName().."Debuff1Count"):SetText(count);
		else
			getglobal(button:GetName().."Debuff1Count"):SetText(nil);
		end

		button.debuff = debuff;
	else
		if ( button.debuff ) then
			getglobal(button:GetName().."Debuff1"):Hide();
			ER_UnitButton_UpdateWidth(button);
			button.debuff = nil;
		end
	end

	if ( debuffIsDispellable ) then
		getglobal(button:GetName().."Debuff1Border"):Show();
	else
		getglobal(button:GetName().."Debuff1Border"):Hide();
	end

	if ( debuffIsDispellable and not button.debuffIsOutOfRange ) then
		if ( not button.debuffIsFlashing ) then
			UIFrameFlash(getglobal(button:GetName().."Debuff1Border"), 0.3, 0.3, -1);
			button.debuffIsFlashing = true;
		end
	else
		if ( button.debuffIsFlashing ) then
			UIFrameFlashStop(getglobal(button:GetName().."Debuff1Border"));
			button.debuffIsFlashing = nil;
		end
	end
end

function ER_UnitButton_UpdateWidth(button)
	local deltaX = 0;
	local deltaWidth = 0;

	if ( (getglobal(button:GetName().."RaidTargetIcon") and getglobal(button:GetName().."RaidTargetIcon"):IsShown()) or
			(getglobal(button:GetName().."FlagIcon") and getglobal(button:GetName().."FlagIcon"):IsShown()) ) then
		deltaWidth = deltaWidth + 14; -- 14
		deltaX = 14; -- 13
	end

	if ( getglobal(button:GetName().."Debuff1"):IsShown() ) then
		deltaWidth = deltaWidth + 14;
	end

	getglobal(button:GetName().."Frame"):SetWidth(103 - deltaWidth/0.75); -- /0.9
	getglobal(button:GetName().."Frame"):SetPoint("BOTTOMLEFT", 8 + deltaX, -3);
	getglobal(button:GetName().."HealthBar"):SetWidth(73 - deltaWidth*0.97); -- *0.93
	getglobal(button:GetName().."HealthBar"):SetPoint("BOTTOMLEFT", 10 + deltaX, 7);
	getglobal(button:GetName().."ManaBar"):SetWidth(73 - deltaWidth*0.97); -- *0.93
end

function ER_UnitButton_UpdateMisc(button)
	if ( UnitIsDeadOrGhost(button.unit) ) then
		ER_UnitButton_UpdateDead(button);
	end
end

-----------------------------------------------------------------------

function ER_BuffButton_OnEnter()
	if ( this:GetCenter() > GetScreenWidth()/2 ) then
		GameTooltip:SetOwner(this, "ANCHOR_LEFT");
	else
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	end

	if ( not this.isDebuff ) then
		GameTooltip:SetUnitBuff(this:GetParent().unit, this.index);
		local timeLeft = ER_RaidAssistGetUnitBuffTimeLeft(this:GetParent().unit, GameTooltipTextLeft1:GetText());
		if ( timeLeft ) then
			GameTooltip:AppendText(" ("..ER_FormatTime(timeLeft)..")");
		end
	else
		GameTooltip:SetUnitDebuff(this.unit, 1, this.isDispellable);
	end	
end

-----------------------------------------------------------------------

function ER_UnitButton_ShowMenu()
	local unit = this.unit;
	local raidIndex = this.raidIndex;

	ER_UnitButtonDropDown.unit = unit;
	ER_UnitButtonDropDown.raidIndex = raidIndex;
	ToggleDropDownMenu(1, nil, ER_UnitButtonDropDown, this, this:GetWidth() + 4, this:GetHeight() + 4);
	if ( DropDownList1:IsShown() ) then
		local screenX, _ = UIParent:GetRight();
		screenX = screenX * UIParent:GetEffectiveScale();
		local frameX, _ = DropDownList1:GetRight();
		frameX = frameX * DropDownList1:GetEffectiveScale();
		if ( frameX >= screenX ) then
			DropDownList1:ClearAllPoints();
			DropDownList1:SetPoint("TOPRIGHT", this, "TOPLEFT", -8, 4);
		end
	end
end

function ER_UnitButtonDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, ER_UnitButtonDropDown_Initialize, "MENU");
end

function ER_UnitButtonDropDown_Initialize()
	local unit = ER_UnitButtonDropDown.unit;
	local raidIndex = ER_UnitButtonDropDown.raidIndex;
	if ( unit and UnitInRaid(unit) and UnitCanCooperate("player", unit) ) then
		ER_unitMenuButtons = {};

		local name, server = UnitName(unit);
		local isMe = UnitIsUnit(unit, "player");
		local _, rank = GetRaidRosterInfo(raidIndex);
	
		local count = 0;
		local info;

		if ( UIDROPDOWNMENU_MENU_LEVEL == 1 ) then
			info = {};
			info.text = UnitName(unit);
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info);
			count = count + 1;
	
			if ( not isMe ) then
				info = {};
				info.text = WHISPER;
				info.value = "WHISPER";
				info.notCheckable = 1;
				if ( not UnitIsConnected(unit) ) then
					info.disabled = 1;
				end
				info.func = function()
					if ( server and not UnitIsSameServer("player", unit)) then
						ChatFrame_SendTell(name.."-"..server);
					else
						ChatFrame_SendTell(name);
					end
				end
				UIDropDownMenu_AddButton(info);
				count = count + 1;
			end
	
			if ( not isMe ) then
				info = {};
				info.text = INSPECT;
				info.value = "INSPECT";
				info.notCheckable = 1;
				if ( UnitIsDeadOrGhost("player") ) then
					info.disabled = 1;
				end
				info.func = function()
					InspectUnit(unit);
				end
				UIDropDownMenu_AddButton(info);	
				count = count + 1;
				ER_unitMenuButtons[info.value] = { id = count, dist = 1 };
			end
	
			if ( not isMe ) then
				info = {};
				info.text = TRADE;
				info.value = "TRADE";
				info.notCheckable = 1;
				if ( UnitIsDeadOrGhost("player") or UnitIsDeadOrGhost(unit) or not HasFullControl() ) then
					info.disabled = 1;
				end
				info.func = function()
					InitiateTrade(unit);
				end
				UIDropDownMenu_AddButton(info);	
				count = count + 1;
				ER_unitMenuButtons[info.value] = { id = count, dist = 2 };
			end
	
			if ( not isMe ) then
				info = {};
				info.text = FOLLOW;
				info.value = "FOLLOW";
				info.notCheckable = 1;
				if ( UnitIsDead("player") ) then
					info.disabled = 1;
				end
				info.func = function()
					FollowUnit(name, 1);
				end
				UIDropDownMenu_AddButton(info);	
				count = count + 1;
				ER_unitMenuButtons[info.value] = { id = count, dist = 4 };
			end
	
			if ( IsRaidOfficer() ) then
				info = {};
				info.text = RAID_TARGET_ICON;
				info.value = "RAID_TARGET_ICON";
				info.notCheckable = 1;
				info.hasArrow = 1;
				UIDropDownMenu_AddButton(info);	
				count = count + 1;
			end
	
			if ( not isMe and IsRaidLeader() and rank ~= 2 ) then
				info = {};
				info.text = SET_RAID_LEADER;
				info.value = "RAID_LEADER";
				info.notCheckable = 1;
				info.func = function()
					PromoteToLeader(name, 1);
				end
				UIDropDownMenu_AddButton(info);	
				count = count + 1;
			end
	
			if ( not isMe and IsRaidLeader() and rank == 0 ) then
				info = {};
				info.text = SET_RAID_ASSISTANT;
				info.value = "RAID_PROMOTE";
				info.notCheckable = 1;
				info.func = function()
					PromoteToAssistant(name, 1);
				end
				UIDropDownMenu_AddButton(info);	
				count = count + 1;
			end
	
			if ( not isMe and IsRaidLeader() and rank == 1 ) then
				info = {};
				info.text = DEMOTE;
				info.value = "RAID_DEMOTE";
				info.notCheckable = 1;
				info.func = function()
					DemoteAssistant(name, 1);
				end
				UIDropDownMenu_AddButton(info);	
				count = count + 1;
			end
	
			local lootMethod, _, masterLootRaidIndex = GetLootMethod();
			if ( lootMethod == "master" ) then
				info = {};
				info.text = LOOT_PROMOTE;
				info.value = "LOOT_PROMOTE";
				info.notCheckable = 1;
				if ( masterLootRaidIndex and unit == "raid"..masterLootRaidIndex ) then
					info.disabled = 1;
				end
				info.func = function()
					SetLootMethod("master", name, 1);
				end
				UIDropDownMenu_AddButton(info);	
				count = count + 1;
			end
	
			if ( not isMe and IsRaidOfficer()) then
				info = {};
				info.text = REMOVE;
				info.value = "RAID_REMOVE";
				info.notCheckable = 1;
				if ( rank == 2 ) then
					info.disabled = 1;
				end
				info.func = function()
					UninviteUnit(name);
				end
				UIDropDownMenu_AddButton(info);	
				count = count + 1;
			end
	
			info = {};
			info.text = CANCEL;
			info.value = "CANCEL";
			info.notCheckable = 1;
			info.func = function()
			end
			UIDropDownMenu_AddButton(info);	
			count = count + 1;
			
			ER_UnitButtonDropDown_UpdateRangeCheck(1);
		else
			local buttons = {
				{ color = {r = 1.0, g = 0.92, b = 0}, tCoordLeft = 0, tCoordRight = 0.25, tCoordTop = 0, tCoordBottom = 0.25 },
				{ color = {r = 0.98, g = 0.57, b = 0}, tCoordLeft = 0.25, tCoordRight = 0.5, tCoordTop = 0, tCoordBottom = 0.25 },
				{ color = {r = 0.83, g = 0.22, b = 0.9}, tCoordLeft = 0.5, tCoordRight = 0.75, tCoordTop = 0, tCoordBottom = 0.25 },
				{ color = {r = 0.04, g = 0.95, b = 0}, tCoordLeft = 0.75, tCoordRight = 1, tCoordTop = 0, tCoordBottom = 0.25 },
				{ color = {r = 0.7, g = 0.82, b = 0.875}, tCoordLeft = 0, tCoordRight = 0.25, tCoordTop = 0.25, tCoordBottom = 0.5 },
				{ color = {r = 0, g = 0.71, b = 1}, tCoordLeft = 0.25, tCoordRight = 0.5, tCoordTop = 0.25, tCoordBottom = 0.5 },
				{ color = {r = 1.0, g = 0.24, b = 0.168}, tCoordLeft = 0.5, tCoordRight = 0.75, tCoordTop = 0.25, tCoordBottom = 0.5 },
				{ color = {r = 0.98, g = 0.98, b = 0.98}, tCoordLeft = 0.75, tCoordRight = 1, tCoordTop = 0.25, tCoordBottom = 0.5 },
			}	
			
			local i;
			for i=1, getn(buttons) do
				local button = buttons[i];
				info = {};
				info.text = getglobal("RAID_TARGET_"..i);
				info.value = "RAID_TARGET_"..i;
				info.textR = button.color.r;
				info.textG = button.color.g;
				info.textB = button.color.b;
				info.icon = "Interface\\TargetingFrame\\UI-RaidTargetingIcons";
				info.tCoordLeft = button.tCoordLeft;
				info.tCoordRight = button.tCoordRight;
				info.tCoordTop = button.tCoordTop;
				info.tCoordBottom = button.tCoordBottom;
				if ( GetRaidTargetIndex(unit) == i ) then
					info.checked = 1;
				end
				info.func = function()
					SetRaidTargetIcon(unit, i)
				end
				UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
				count = count + 1;
			end
			
			info = {};
			info.text = NONE;
			info.value = "RAID_TARGET_NONE";
			info.func = function()
				SetRaidTargetIcon(unit, 0)
			end
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
			count = count + 1;
		end
	end
end

function ER_UnitButtonDropDown_UpdateRangeCheck(forced)
	if ( not forced ) then
		if ( not (DropDownList1:IsShown() and UIDropDownMenu_GetCurrentDropDown() == ER_UnitButtonDropDown) ) then
			return;
		end
	end
	
	if ( not ER_unitMenuButtons or not ER_UnitButtonDropDown.unit ) then
		return;
	end
	
	local index, value;
	for index, value in pairs(ER_unitMenuButtons) do
		if ( value.dist ) then
			if ( CheckInteractDistance(ER_UnitButtonDropDown.unit, value.dist) ) then
				UIDropDownMenu_EnableButton(1, value.id);
			else
				UIDropDownMenu_DisableButton(1, value.id);
			end
		end
	end
end